#!/bin/bash
#SBATCH -J pgd_prep
#SBATCH -p normal64
#SBATCH -N 2
#SBATCH -n 80
#SBATCH -t 24:00:00
#SBATCH -o pgd_prep.log
#SBATCH -e pgd_prep.log
#=========================================================================
#
# Script to compute the PGD and PREP files (including oasis and trip prep files)
# Optionnaly also compute the PGD_fractown
# All files are saved in the pgd_prep subdirectory
#
# USAGE
#   sbatch ./job_pgd_prep VERSION LPGD LFRACTOWN LPREP
#
# by default: LPGD=1, LFRACTOWN=1 and LPREP=1
#
#=========================================================================
#set -x
#
VERSION=${1:--X0}

LPGD=${2:-1}
LFRACTOWN=${3:-1}
LPREP=${4:-1}


#
RUNDIR=$(pwd)
EXPNAME=pgd_prep
#
# check if TRIP is used
LTRIP=`grep "LOASIS *= *T" OPTIONS/OPTIONS.nam | wc -l`
#
SURFEXNAML=OPTIONS.nam
[ $LTRIP -eq 1 ] && TRIPNAML=TRIP_OPTIONS.nam
[ $LTRIP -eq 1 ] && OASISNAML=namcouple
#
#=========================================================================
# ECOCLIMAP AND SURFEX 
#=========================================================================
#
ROOT_DIR=/home/$(whoami)/LDAS
SURF_DIR=$ROOT_DIR/SURFEX
DATA_DIR=$ROOT_DIR/Data
ECODIR=$DATA_DIR/ECOCLIMAP
[ $LTRIP -eq 1 ] && TRIPDIR=$DATA_DIR/ECOCLIMAP/TRIP
#
VER=`find $SURF_DIR/exe -name PGD-*${VERSION}`
VER=`basename $VER`
VER=${VER:4}

EXEPGD=$SURF_DIR/exe/PGD-$VER
EXEPREP=$SURF_DIR/exe/PREP-$VER
[ $LTRIP -eq 1 ] && EXETRIP=$SURF_DIR/exe/TRIP_PREP-$VER
#
#=========================================================================
# DIRECTORIES AND INIT RUN
#=========================================================================
#
SAVEDIR=$RUNDIR/$EXPNAME
FTDIR=${FTDIR:-tmp}
#
mkdir -p $SAVEDIR
mkdir -p $FTDIR/$EXPNAME
cd $FTDIR/$EXPNAME
rm -f *
#
#=========================================================================
#
ulimit -s unlimited
unset I_MPI_PMI_LIBRARY
#
export OMP_NUM_THREADS=1
export DR_HOOK=0
export DR_HOOK_OPT=prof
#
#=========================================================================
# NAMELIST
#=========================================================================
#
cat $RUNDIR/OPTIONS/CSELECT.OFF0 $RUNDIR/OPTIONS/$SURFEXNAML > OPTIONS.nam
$RUNDIR/SCRIPTS/SET_OPTIONS.sh
#
GRIDTYP=`grep 'CGRID*=*' OPTIONS.nam | cut -d= -f2`
GRIDTYP=${GRIDTYP%\'*}
GRIDTYP=${GRIDTYP:(-3)}
[ "$GRIDTYP" = "LONLATVAL" ] && sed -i "/YALG_MPI.*=/cYALG_MPI = 'LIN'," OPTIONS.nam
#
[ $LTRIP -eq 1 ] && cp $RUNDIR/OPTIONS/$TRIPNAML TRIP_OPTIONS.nam
[ $LTRIP -eq 1 ] && cp $RUNDIR/OPTIONS/$OASISNAML namcouple
[ $LTRIP -eq 1 ] && sed -i "s/XXXXXX/86400/" namcouple
#
ln -s $ECODIR/ecoclimapI_covers_param.bin .
ln -s $ECODIR/ecoclimapII_eu_covers_param.bin .
#
#=========================================================================
# PGD CASE
#=========================================================================
#
if [[ $LPGD -eq 1 ]] || [[ $LFRACTOWN -eq 1 ]] || [[ ! -f $SAVEDIR/PGD.nc ]]; then
  echo " !!!!!!!!!! PGD CASE !!!!!!!! "
  #
  ln -s $ECODIR/gtopo30.* .
  ln -s $ECODIR/topo_index.* .
  ln -s $ECODIR/soc* .
  ln -s $ECODIR/perm_glo* .
  #ln -s $ECODIR/groudwater* .
  #
  ln -s $ECODIR/ECOCLIMAP_II_EUROP_V2.3.dir ecoclimap.dir
  ln -s $ECODIR/ECOCLIMAP_II_EUROP_V2.3.hdr ecoclimap.hdr
  #
  ln -s $ECODIR/LAKE_DEPTH_ECO_II_V2.3.dir lake_depth.dir
  ln -s $ECODIR/LAKE_DEPTH_ECO_II_V2.3.hdr lake_depth.hdr
  #
  ln -s $ECODIR/CLAY_HWSD_MOY_v2.dir clay.dir
  ln -s $ECODIR/CLAY_HWSD_MOY_v2.hdr clay.hdr
  ln -s $ECODIR/SAND_HWSD_MOY_v2.dir sand.dir
  ln -s $ECODIR/SAND_HWSD_MOY_v2.hdr sand.hdr
  #
  ln -s $EXEPGD pgd.exe
  #
  if [ $LPGD -eq 1 ]; then
    mpirun -np $SLURM_NTASKS pgd.exe
    if [[ $? -ne 0 ]] ; then
       echo "run pgd failed !!! See in directory: $FTDIR/$EXPNAME"
       exit 1
    fi
    mv PGD.nc $SAVEDIR/PGD.nc
  fi
  #
  if [ $LFRACTOWN -eq 1 ]; then
    sed -i "/LTOWN_TO_ROCK/c\    LTOWN_TO_ROCK = F" OPTIONS.nam
    mpirun -np $SLURM_NTASKS pgd.exe
    if [[ $? -ne 0 ]] ; then
       echo "run pgd failed !!! See in directory: $FTDIR/$EXPNAME"
       exit 1
    fi
    mv PGD.nc $SAVEDIR/PGD_fractown.nc
    sed -i "/LTOWN_TO_ROCK/c\    LTOWN_TO_ROCK = T" OPTIONS.nam
  fi
  #
fi
#
#=========================================================================
# PREP CASE
#=========================================================================
#
if [[ $LPREP -eq 1 ]]; then
  echo " !!!!!!!!!! PREP CASE !!!!!!!! "
  #
  ln -s $SAVEDIR/PGD.nc PGD.nc
  #
  if [ $LTRIP -eq 1 ]; then
    ln -s $TRIPDIR/TRIP_PGD_HD.nc .
    ln -s $TRIPDIR/TRIP_SGAQUI_HD.nc .
    ln -s $TRIPDIR/TRIP_SGFLOOD_HD.nc .
    ln -s $TRIPDIR/TRIP_INIT_HD.nc .
  fi
  #
  ln -s $EXEPREP prep.exe
  [ $LTRIP -eq 1 ] && ln -s $EXETRIP trip_prep.exe
  #
  if [ $LTRIP -eq 1 ]; then
    mpirun -np $((SLURM_NTASKS-1)) prep.exe : -np 1 trip_prep.exe
  else
    mpirun -np $SLURM_NTASKS prep.exe
  fi
  #
  if [[ $? -ne 0 ]] ; then
    echo "run prep failed !!! See in directory: $FTDIR/$EXPNAME"
    exit
  fi
  #
  if [[ $DR_HOOK -ne 0 ]] ; then
    mv drhook.prof.1 $SAVEDIR/drhook.prof_prep
    mv LISTING_PREP.txt $SAVEDIR/LISTING_PREP.txt
  fi
  #
  YY=`grep NYEAR OPTIONS.nam | sed -e "s/.*= \([0-9]*\) *,/\1/"`
  MM=`grep NMONTH OPTIONS.nam | sed -e "s/.*= \([0-9]*\) *,/\1/"`
  DD=`grep NDAY OPTIONS.nam | sed -e "s/.*= \([0-9]*\) *,/\1/"`
  HH=$((`grep XTIME OPTIONS.nam | sed -e "s/.*= \([0-9]*\). *,/\1/"`/3600))
  printf -v PREP_DATE "%04g%02g%02g%02g" $YY $MM $DD $HH
  #
  mv PREP.nc $SAVEDIR/PREP_${PREP_DATE}.nc
  if [ $LTRIP -eq 1 ]; then
    mv TRIP_PREP.nc $SAVEDIR/TRIP_PREP_${PREP_DATE}.nc
    mv TRIP_PARAM.nc $SAVEDIR/.
    mv areas.nc $SAVEDIR/.
    mv grids.nc $SAVEDIR/.
    mv masks.nc $SAVEDIR/.
    # just do it here. tell me if it does not work
    cd $RUNDIR
    python ./SCRIPTS/OASIS_RESTART.py $EXPNAME $PREP_DATE $GRIDTYP
    cd -
  fi
  #
fi
#
rm -rf $FTDIR/$EXPNAME
#
#=========================================================================
exit 0
#=========================================================================
