#!/bin/bash
#SBATCH -J spinup
#SBATCH -p normal64
#SBATCH -N 5
#SBATCH -n 200
#SBATCH -t 24:00:00
#SBATCH -o spinup.log
#SBATCH -e spinup.log
#=========================================================================
#
# Script to run a SURFEX experiment N times over the same year
# Prep files are saved in the pgd_prep and pgd_prep/$EXP subdirectory
# Initial prep files in pgd_prep directory must have the form PREP_YYYYMMDDHH_INIT.nc
# Monthly forcing files of the year corresponding to PREP files must be in forcings subdirectory
#
# USAGE
#   nohup ./job_spinup DOM[:EXP] nodes[:tasks_per_node:tasks_for_trip N START_ID:START_MM] &
#
# by default: EXP=spinup, N=20, START_ID=0, START_MM=1
#
#=========================================================================
#set -x
#
if [ $# -lt 2 ]; then
  echo "Usage: $0 DOM[:EXP:VERSION] nodes[:tasks_per_node:tasks_for_trip N START_ID:START_MM]"
  echo "DOM            = the name of a domain"
  echo "EXP            = the name of an experiment"
  echo "N              = the number of runnings"
  echo "START_ID       = start id of a running"
  echo "START_MM       = start month of a running"
  exit
fi

DOM=`echo $1 | awk '{split($1, f, ":"); print f[1]}'`
EXP=`echo $1 | awk '{split($1, f, ":"); print f[2]}'`
EXP=${EXP:-spinup}
VERSION=`echo $1 | awk '{split($1, f, ":"); print f[3]}'`
VERSION=${VERSION:--X0}
num_nodes=`echo ${2} | awk '{split($1, f, ":"); print f[1]}'`
tasks_per_node=`echo ${2} | awk '{split($1, f, ":"); print f[2]}'`
tasks_per_node=${tasks_per_node:-40}
tasks_for_trip=`echo ${2} | awk '{split($1, f, ":"); print f[3]}'`
tasks_for_trip=${tasks_for_trip:-1}
SPINUP_N=${3:-20}
START_ID=`echo $4 | awk '{split($1, f, ":"); print f[1]}'`
START_ID=${START_ID:-0}
START_MM=`echo $4 | awk '{split($1, f, ":"); print f[2]}'`
START_MM=${START_MM:-1}

#
# check if TRIP is used
LTRIP=`grep "LOASIS *= *T" OPTIONS/OPTIONS.nam | wc -l`
#
ulimit -s unlimited
unset I_MPI_PMI_LIBRARY
export OMP_NUM_THREADS=1
export DR_HOOK=0
export DR_HOOK_OPT=prof
#
#=========================================================================
# ECOCLIMAP AND SURFEX 
#=========================================================================
#
FORCINGDIR=/home/$(whoami)/LDAS/$DOM/forcings/monthly #on hendrix
ROOT_DIR=/home/$(whoami)/LDAS
SURF_DIR=$ROOT_DIR/SURFEX
DATA_DIR=$ROOT_DIR/Data
ECODIR=$DATA_DIR/ECOCLIMAP
#
VER=`find $SURF_DIR/exe -name PGD-*${VERSION}`
VER=`basename $VER`
VER=${VER:4}

SFXEXE=$SURF_DIR/exe/OFFLINE-$VER
[ $LTRIP -eq 1 ] && RIVEXE=$SURF_DIR/exe/TRIP_MASTER-$VER
#
#=========================================================================
# DIRECTORIES AND LINKS
#=========================================================================
#
RUNDIR=$(pwd)
[ ! -f EXP/$EXP ] && mkdir -p EXP/$EXP
[ ! -f $RUNDIR/pgd_prep/$EXP ] && mkdir -p $RUNDIR/pgd_prep/$EXP
cd EXP/$EXP
rm -f [^FORCING]*
#
ln -sf $SFXEXE offline.exe
[ $LTRIP -eq 1 ] && ln -sf $RIVEXE trip.exe
#
# get params
ln -s $ECODIR/ecoclimapI_covers_param.bin
ln -s $ECODIR/ecoclimapII_eu_covers_param.bin
ln -s $RUNDIR/pgd_prep/PGD.nc PGD.nc
[ $LTRIP -eq 1 ] && ln -s $RUNDIR/pgd_prep/TRIP_PARAM.nc
[ $LTRIP -eq 1 ] && ln -s $RUNDIR/pgd_prep/areas.nc
[ $LTRIP -eq 1 ] && ln -s $RUNDIR/pgd_prep/grids.nc
[ $LTRIP -eq 1 ] && ln -s $RUNDIR/pgd_prep/masks.nc
OASISDIR=$RUNDIR/data_oasis
LRMP=`( [ -d $OASISDIR ] && find $OASISDIR -name rmp*.nc ) | wc -l`
[ $LTRIP -eq 1 ] && [ $LRMP -gt 0 ] && ln -s $OASISDIR/rmp*.nc .
#
# get restart
if [ $START_ID -eq 0 ] && [ $START_MM -eq 1 ]; then
  rm -f $RUNDIR/pgd_prep/$EXP/PROF.dat
  PREPFILE=`find $RUNDIR/pgd_prep -name PREP_*_INIT.nc`
  PREPFILE=`basename $PREPFILE`
  YYYYMMDDHH=${PREPFILE:5:10}
  ln -sf $RUNDIR/pgd_prep/PREP_${YYYYMMDDHH}_INIT.nc PREP.nc
  [ $LTRIP -eq 1 ] && ln -sf $RUNDIR/pgd_prep/TRIP_PREP_${YYYYMMDDHH}_INIT.nc TRIP_PREP.nc
  [ $LTRIP -eq 1 ] && cp $RUNDIR/pgd_prep/lsm_${YYYYMMDDHH}_INIT.nc lsm.nc
  [ $LTRIP -eq 1 ] && cp $RUNDIR/pgd_prep/trip_${YYYYMMDDHH}_INIT.nc trip.nc
else
  if [ $START_MM -eq 1 ]; then
    FID=$((START_ID - 1))
  else
    FID=$START_ID
  fi
  ln -sf $RUNDIR/pgd_prep/$EXP/PREP_$FID.nc PREP.nc
  ln -sf $RUNDIR/pgd_prep/$EXP/TRIP_PREP_$FID.nc TRIP_PREP.nc
  cp $RUNDIR/pgd_prep/$EXP/lsm_$FID.nc lsm.nc
  cp $RUNDIR/pgd_prep/$EXP/trip_$FID.nc trip.nc
fi
YEAR=`ncdump -v DTCUR-YEAR PREP.nc | grep 'DTCUR-YEAR = ' | awk '{print $3}'`
MONTH=`ncdump -v DTCUR-MONTH PREP.nc | grep 'DTCUR-MONTH = ' | awk '{print $3}'`
DAY=`ncdump -v DTCUR-DAY PREP.nc | grep 'DTCUR-DAY = ' | awk '{print $3}'`
TIME=`ncdump -v DTCUR-TIME PREP.nc | grep 'DTCUR-TIME = ' | awk '{print $3}'`
YYYYMMDDHH=`echo "$YEAR $MONTH $DAY $TIME" | awk '{printf "%4d%02d%02d%02d",$1,$2,$3,$4/3600}'`
( [ ! -f PREP.nc ] || ( [ $LTRIP -eq 1 ] && ( [ ! -f TRIP_PREP.nc ] || \
  [ ! -f lsm.nc ] || [ ! -f trip.nc ] ) ) ) && \
  echo "Some PREP files are missing" && exit 1
#
# get option files
cat $RUNDIR/OPTIONS/CSELECT.PERT $RUNDIR/OPTIONS/OPTIONS.nam > OPTIONS.nam
$RUNDIR/SCRIPTS/SET_OPTIONS.sh OPTIONS.nam
#
GRIDTYP=`grep 'CGRID*=*' OPTIONS.nam | cut -d= -f2`
GRIDTYP=${GRIDTYP%\'*}
GRIDTYP=${GRIDTYP:(-3)}
[ "$GRIDTYP" = "LONLATVAL" ] && sed -i "/YALG_MPI.*=/cYALG_MPI = 'LIN'," OPTIONS.nam
#
[ $LTRIP -eq 1 ] && cp -f $RUNDIR/OPTIONS/TRIP_OPTIONS.nam .
[ $LTRIP -eq 1 ] && cp -f $RUNDIR/OPTIONS/namcouple namcouple_xxx
#
#=========================================================================
# LOOP
#=========================================================================
#
echo "***************************************"
echo "   READY TO START SPINUP on $(hostname)"
echo "     $EXP $SPINUP_N $START_ID:$START_MM"
echo "***************************************"
date > date_$EXP

hn=`hostname`
if [ "${hn::12}" = "beaufixlogin" ]; then
  YYYY=${YYYYMMDDHH::4}
  for MM in `seq -w 1 12`
  do
    [ ! -f FORCING_${YYYY}${MM}.nc ] && lftp hendrix -e "get -c $FORCINGDIR/FORCING_${YYYY}${MM}.nc; quit"
  done
  cd $RUNDIR
  rm -f $RUNDIR/spinup.log
  sed -i "/^#SBATCH -N/c\#SBATCH -N ${num_nodes}" $0
  sed -i "/^#SBATCH -n/c\#SBATCH -n $((num_nodes*tasks_per_node))" $0
  DT=10

  jobid=''; while [ "$jobid" = "" ]; do
    jobid=`sbatch $0 $DOM:$EXP ${num_nodes}:${tasks_per_node}:${tasks_for_trip} $SPINUP_N $START_ID:$START_MM`
    sleep $DT
  done
  echo "$jobid: $0 $DOM:$EXP ${num_nodes}:${tasks_per_node}:${tasks_for_trip} $SPINUP_N $START_ID:$START_MM"
  jobid=`echo $jobid | awk '{print $4}'`
  
  # check if batch runs completed
  while true; do
    sleep $DT
    
    if [ -f $RUNDIR/spinup.log ]; then
      STATE1=`grep 'State                : ' $RUNDIR/spinup.log`
      STATE2=`echo "$STATE1" | grep 'State                : COMPLETED'`
      STATE3=`echo "$STATE1" | grep 'State                : '`
      STATE1=`grep 'DUE TO TIME LIMIT' $RUNDIR/spinup.log`
        
      [ ! -z "$STATE2" ] && exit
      if [ ! -z "$STATE3" ] || [ ! -z "$STATE1" ]; then
        scancel $jobid 2> /dev/null
        while [ ! -z "`squeue --job $jobid 2> /dev/null | grep $jobid`" ]; do sleep $DT; done

        if [ -f $RUNDIR/pgd_prep/$EXP/.donelist ]; then
          START_ID=`cat $RUNDIR/pgd_prep/$EXP/.donelist | awk '{print $1}'`
          START_MM=`cat $RUNDIR/pgd_prep/$EXP/.donelist | awk '{print $2}'`
          sleep $DT; mv $RUNDIR/spinup.log $RUNDIR/spinup_${START_ID}.${YYYY}${START_MM}.log
          if [ $START_MM -eq 12 ]; then
            START_MM=1
            START_ID=$((START_ID+1))
          else
            START_MM=$((10#$START_MM+1))
          fi

          jobid=''; while [ "$jobid" = "" ]; do
            jobid=`sbatch $0 $DOM:$EXP ${num_nodes}:${tasks_per_node}:${tasks_for_trip} $SPINUP_N $START_ID:$START_MM`
            sleep $DT
          done
          echo "$jobid: $0 $DOM:$EXP ${num_nodes}:${tasks_per_node}:${tasks_for_trip} $SPINUP_N $START_ID:$START_MM"
          jobid=`echo $jobid | awk '{print $4}'`
        fi
      fi
    fi
  done
fi
rm -f $RUNDIR/pgd_prep/$EXP/.donelist

YYYY=${YYYYMMDDHH::4}
MM=${YYYYMMDDHH:4:2}
j=$START_ID
while [ $j -lt $SPINUP_N ] ; do

  echo " "
  echo "------------------"
  echo "SPINUP : $j / $SPINUP_N"

  while [ $MM -le 12 ] ; do

    echo "    $YYYY$MM"
    # get forcing for this month
    ln -sf FORCING_${YYYY}${MM}.nc FORCING.nc
    $RUNDIR/SCRIPTS/CHECK_DATES.sh || exit 1

    # adjust run length in namcouple
    TEND=`ncdump -v time FORCING.nc | sed -n '1,/data:/d;p' | \
        sed ':a;N;$!ba;s/\n/ /g' | awk '{print 3600*($(NF-2)-$3)}'`
    [ $LTRIP -eq 1 ] && sed -e "s/XXXXXX/$TEND/" namcouple_xxx >  namcouple

    # run OFFLINE
    time_beg=`date +%s` 
    if [ $LTRIP -eq 1 ]; then
      mpirun -np $((SLURM_NTASKS - tasks_for_trip)) offline.exe : -np $tasks_for_trip trip.exe &> offline
    else
      mpirun -np ${SLURM_NTASKS} offline.exe &> offline
    fi
    [ $? -gt 0 ] && echo "Error with offline" && exit 1
    time_end=`date +%s` 
    echo "${YYYY}${MM} $j $((time_end - time_beg))" >> $RUNDIR/pgd_prep/$EXP/PROF.dat
 
    # surfout analysed file is prep for next day
    mv SURFOUT.nc PREP.nc
    [ $LTRIP -eq 1 ] && mv TRIP_RESTART.nc TRIP_PREP.nc

    # Change dates to start again
    if [ $MM -eq 12 ]; then
      ncap2 -O -s "'DTCUR-YEAR'=$YYYY;'DTCUR-MONTH'=1;'DTCUR-DAY'=1;'DTCUR-TIME'=0" PREP.nc PREP.nc
      [ $LTRIP -eq 1 ] && ncap2 -O -s "date(:)={$YYYY,1,1,0}" TRIP_PREP.nc TRIP_PREP.nc
    fi

    # backup files for restart
    cp -f PREP.nc $RUNDIR/pgd_prep/$EXP/PREP_${j}.nc
    if [ $LTRIP -eq 1 ]; then
      cp -f TRIP_PREP.nc $RUNDIR/pgd_prep/$EXP/TRIP_PREP_${j}.nc
      cp -f lsm.nc $RUNDIR/pgd_prep/$EXP/lsm_${j}.nc
      cp -f trip.nc $RUNDIR/pgd_prep/$EXP/trip_${j}.nc
      [ $LRMP -eq 0 ] && mkdir -p $OASISDIR && cp -f rmp*.nc $OASISDIR && LRMP=1
    fi
    echo "$j $MM" > $RUNDIR/pgd_prep/$EXP/.donelist
    
    # increment date of 1 month
    MM=$( printf "%02d" $(( 10#$MM + 1 )) )
  done

  echo '------------------'
  echo ' '

  MM=01
  j=$(( j+1 ))

done

# save prep files
YEAR=`ncdump -v DTCUR-YEAR PREP.nc | grep 'DTCUR-YEAR = ' | awk '{print $3}'`
MONTH=`ncdump -v DTCUR-MONTH PREP.nc | grep 'DTCUR-MONTH = ' | awk '{print $3}'`
DAY=`ncdump -v DTCUR-DAY PREP.nc | grep 'DTCUR-DAY = ' | awk '{print $3}'`
TIME=`ncdump -v DTCUR-TIME PREP.nc | grep 'DTCUR-TIME = ' | awk '{print $3}'`
YYYYMMDDHH=`echo "$YEAR $MONTH $DAY $TIME" | awk '{printf "%4d%02d%02d%02d",$1,$2,$3,$4/3600}'`

mv PREP.nc $RUNDIR/pgd_prep/PREP_${YYYYMMDDHH}.nc
[ $LTRIP -eq 1 ] && mv TRIP_PREP.nc $RUNDIR/pgd_prep/TRIP_PREP_${YYYYMMDDHH}.nc
[ $LTRIP -eq 1 ] && mv lsm.nc $RUNDIR/pgd_prep/lsm_${YYYYMMDDHH}.nc
[ $LTRIP -eq 1 ] && mv trip.nc $RUNDIR/pgd_prep/trip_${YYYYMMDDHH}.nc

# save rmp files for next run
[ $LTRIP -eq 1 ] && [ $LRMP -eq 0 ] && mkdir -p $OASISDIR && mv rmp*.nc $OASISDIR

# end simulation
date >> date_$EXP
echo "***************************************"
echo "   SPINUP ENDS CORRECTLY"
echo "***************************************"
