#!/bin/bash
#======================================================================================
#
# Script to run SURFEX-TRIP open-loop, closed-loop and assimilating daily observations
#
#======================================================================================
#set -x
if [ $# -lt 5 ]; then
  echo "Usage: $0 TYP[:NJOB:days_FC] DOM:EXP YYYYMMDDHH_beg[:YYYYMMDDHH_end] days_per_job[:days_per_ftp] num_nodes[:tasks_per_node:tasks_for_trip] [job_time_limit] [VERSION]"
  echo "TYP            = ENKF if run the ensemble of SURFEX then an Ensemble Kalman filter (closed-loop)"
  echo "               = EKF if run the perturbations of SURFEX then an Extended Kalman filter (closed-loop)"
  echo "               = OL if run the offline SURFEX without assimilations (open-loop)"
  echo "               = ENOL if run the ensemble offline SURFEX without assimilations (open-loop)"
  echo "NJOB           = the number of jobs submited at the same time for the ensemble or perturbations"
  echo "days_FC        = the days of a deterministic forecast, zero if no deterministic forecsts"
  echo "DOM            = the name of a domain"
  echo "EXP            = the name of an experiment"
  echo "job_time_limit = hh:mm:ss"
  echo "VERSION        = not specified if the standard version is used"
  echo "               = MYSRC_EnSRF for example, if the user verion is used"
  exit
fi

TYP=`echo $1 | awk '{split($1, f, ":"); print f[1]}'`
NJOB=`echo $1 | awk '{split($1, f, ":"); print f[2]}'`
NJOB=${NJOB:-0}
days_FC=`echo $1 | awk '{split($1, f, ":"); print f[3]}'`
days_FC=${days_FC:-0}

DOM=`echo $2 | awk '{split($1, f, ":"); print f[1]}'`
EXP=`echo $2 | awk '{split($1, f, ":"); print f[2]}'`
[ -z "$EXP" ] && echo "You must specify the name of an experiment" && exit 1

YYYYMMDDHH_beg=`echo $3 | awk '{split($1, f, ":"); print f[1]}'`
YYYYMMDDHH_end=`echo $3 | awk '{split($1, f, ":"); print f[2]}'`
YYYYMMDDHH_end=${YYYYMMDDHH_end:-$YYYYMMDDHH_beg}

days_per_job=`echo $4 | awk '{split($1, f, ":"); print f[1]}'`
days_per_ftp=`echo $4 | awk '{split($1, f, ":"); print f[2]}'`
days_per_ftp=${days_per_ftp:-30}
[ $((days_per_ftp % days_per_job)) -ne 0 ] && "days_per_ftp must be a multiple of days_per_job" && exit 1

num_nodes=`echo ${5} | awk '{split($1, f, ":"); print f[1]}'`
tasks_per_node=`echo ${5} | awk '{split($1, f, ":"); print f[2]}'`
tasks_per_node=${tasks_per_node:-40}
tasks_for_trip=`echo ${5} | awk '{split($1, f, ":"); print f[3]}'`
tasks_for_trip=${tasks_for_trip:-1}

job_time_limit=${6:-24:00:00}

VERSION=${7:--X0}

#============================================
# check if TRIP is used
#============================================
LTRIP=`grep "LOASIS *= *T" OPTIONS/OPTIONS.nam | wc -l`

#****************************************************
# PLEASE ONLY CHANGE THE DIRECTORIES IN THIS SECTION
#****************************************************
ROOT_DIR=/home/$(whoami)/LDAS
SURF_DIR=$ROOT_DIR/SURFEX
DATA_DIR=$ROOT_DIR/Data
ECODIR=$DATA_DIR/ECOCLIMAP

VER=`find $SURF_DIR/exe -name PGD-*${VERSION}`
VER=`basename $VER`
VER=${VER:4}

SFXEXE=$SURF_DIR/exe/OFFLINE-$VER
[ $LTRIP -eq 1 ] && RIVEXE=$SURF_DIR/exe/TRIP_MASTER-$VER
SODEXE=$SURF_DIR/exe/SODA-$VER

SCRATCHDIR=/scratch/work/$(whoami)/LDAS/$DOM/$EXP #on beaufix/prolix
FORCINGDIR=/home/$(whoami)/LDAS/$DOM/forcings/daily #on hendrix
OBSDIR=/home/$(whoami)/LDAS/$DOM/observations/CANARI #on hendrix
RESDIR=/home/$(whoami)/LDAS/$DOM/$EXP #on hendrix
INIDIR_ENOL=/home/$(whoami)/LDAS/$DOM/enkf #on hendrix

#============================================
# create directories and copy files
#============================================
RUNDIR=$(pwd)
mkdir -p $SCRATCHDIR
rm -rf EXP/$EXP
mkdir -p EXP/$EXP/oe
cd EXP/$EXP

OEDIR=$RUNDIR/EXP/$EXP/oe

# get params
ln -s $ECODIR/ecoclimapI_covers_param.bin .
ln -s $ECODIR/ecoclimapII_eu_covers_param.bin .
ln -s $RUNDIR/pgd_prep/PGD.nc PGD.nc
if [ $LTRIP -eq 1 ]; then
  ln -s $RUNDIR/pgd_prep/TRIP_PARAM.nc .
  ln -s $RUNDIR/pgd_prep/areas.nc .
  ln -s $RUNDIR/pgd_prep/grids.nc .
  ln -s $RUNDIR/pgd_prep/masks.nc .
fi
# get option files
cp -f $RUNDIR/OPTIONS/OPTIONS.nam .
cp -f $RUNDIR/OPTIONS/CSELECT* .
if [ $LTRIP -eq 1 ]; then
  cp -f $RUNDIR/OPTIONS/TRIP_OPTIONS.nam .
  cp -f $RUNDIR/OPTIONS/namcouple namcouple_xxx
fi
# get executables and scripts
cp -f $RUNDIR/SCRIPTS/BATCH.EXE .
sed -i "/^#SBATCH --account=/c\#\#SBATCH --account=" BATCH.EXE
sed -i "/^#SBATCH -J/c\#SBATCH -J batch_${EXP}" BATCH.EXE
sed -i "/^#SBATCH -N/c\#SBATCH -N ${num_nodes}" BATCH.EXE
sed -i "/^#SBATCH -n/c\#SBATCH -n $((num_nodes*tasks_per_node))" BATCH.EXE
sed -i "/^#SBATCH -t/c\#SBATCH -t ${job_time_limit}" BATCH.EXE
cp -f $RUNDIR/SCRIPTS/FORECAST.EXE .
sed -i "/^#SBATCH --account=/c\#\#SBATCH --account=" FORECAST.EXE
sed -i "/^#SBATCH -J/c\#SBATCH -J forecast_${EXP}" FORECAST.EXE
sed -i "/^#SBATCH -N/c\#SBATCH -N ${num_nodes}" FORECAST.EXE
sed -i "/^#SBATCH -n/c\#SBATCH -n $((num_nodes*tasks_per_node))" FORECAST.EXE
sed -i "/^#SBATCH -t/c\#SBATCH -t ${job_time_limit}" FORECAST.EXE
sed -i "/^#SBATCH -o/c\#SBATCH -o $OEDIR/forecast_%j" FORECAST.EXE
sed -i "/^#SBATCH -e/c\#SBATCH -e $OEDIR/forecast_%j" FORECAST.EXE
ln -s $RUNDIR/SCRIPTS/CHECK_DATES.sh .
ln -s $RUNDIR/SCRIPTS/SET_OPTIONS.sh .
ln -s $RUNDIR/SCRIPTS/GET_FORCING.sh .
ln -s $RUNDIR/SCRIPTS/GET_OBSERVATION.sh .
ln -s $RUNDIR/SCRIPTS/GET_RESTART.sh .
ln -s $RUNDIR/SCRIPTS/PUT_RESULT.sh .
ln -s $RUNDIR/SCRIPTS/ADD_DATE.sh .
ln -s $SFXEXE offline.exe
[ $LTRIP -eq 1 ] && ln -s $RIVEXE trip.exe
ln -s $SODEXE soda.exe

# prepare experiment
if [ "$TYP" = "EKF" ] || [ "$TYP" = "ENKF" ]; then
  sed -i "/CASSIM_ISBA.*=/c                       CASSIM_ISBA   = '$TYP'," OPTIONS.nam
fi
#
GRIDTYP=`grep 'CGRID*=*' OPTIONS.nam | cut -d= -f2`
GRIDTYP=${GRIDTYP%\'*}
GRIDTYP=${GRIDTYP:(-3)}
[ "$GRIDTYP" = "LONLATVAL" ] && sed -i "/YALG_MPI.*=/cYALG_MPI = 'LIN'," OPTIONS.nam
#
cat CSELECT.OFF0 OPTIONS.nam > OPTIONS.nam_0
./SET_OPTIONS.sh OPTIONS.nam_0
if [ "$TYP" = "OL" ]; then
  BEG_ID=0
  END_ID=0
  NJOB=1
elif [ "$TYP" = "EKF" ]; then
  BEG_ID=0
  END_ID=`grep NVAR OPTIONS.nam | cut -d= -f2 | sed -e 's/,//' | awk '{print $1}'`
  [ "$END_ID" = "" ] && echo "You need to add the options for EKF in OPTIONS.nam" && exit
  IRUN=1
  while [ $IRUN -le $END_ID ]; do
    cat CSELECT.PERT OPTIONS.nam > OPTIONS.nam_$IRUN
    ./SET_OPTIONS.sh OPTIONS.nam_$IRUN $TYP F T $IRUN PREP
    IRUN=$((IRUN+1))
  done
  [ $NJOB -eq 0 ] && NJOB=$((END_ID+2))
  [ $NJOB -gt $((END_ID+2)) ] && NJOB=$((END_ID+2))
elif [ "$TYP" = "ENKF" ]; then
  BEG_ID=1
  END_ID=`grep NENS OPTIONS.nam | cut -d= -f2 | sed -e 's/,//' | awk '{print $1}'`
  [ "$END_ID" = "" ] && echo "You need to add the options for ENKF in OPTIONS.nam" && exit
  IRUN=1
  while [ $IRUN -le $END_ID ]; do
    cat CSELECT.OFF0 OPTIONS.nam > OPTIONS.nam_$IRUN
    ./SET_OPTIONS.sh OPTIONS.nam_$IRUN $TYP F T $IRUN PREP
    IRUN=$((IRUN+1))
  done
  [ $NJOB -eq 0 ] && NJOB=$((END_ID+1))
  [ $NJOB -gt $((END_ID+1)) ] && NJOB=$((END_ID+1))
elif [ "$TYP" = "ENOL" ]; then
  BEG_ID=1
  END_ID=`grep NENS OPTIONS.nam | cut -d= -f2 | sed -e 's/,//' | awk '{print $1}'`
  [ "$END_ID" = "" ] && echo "You need to add the options for ENOL in OPTIONS.nam" && exit
  IRUN=1
  while [ $IRUN -le $END_ID ]; do
    cat CSELECT.OFF0 OPTIONS.nam > OPTIONS.nam_$IRUN
    ./SET_OPTIONS.sh OPTIONS.nam_$IRUN
    IRUN=$((IRUN+1))
  done
  [ $NJOB -eq 0 ] && NJOB=$((END_ID+1))
  [ $NJOB -gt $((END_ID+1)) ] && NJOB=$((END_ID+1))
fi

# get restart files
./GET_RESTART.sh $BEG_ID:$END_ID $YYYYMMDDHH_beg $TYP:$LTRIP $RESDIR $RUNDIR

#============================================
# loop over dates
#============================================
echo "******************************************************************"
echo " READY TO START SIMULATION ($TYP)"
echo " $YYYYMMDDHH_beg  ->  $YYYYMMDDHH_end"
echo " the script_ldas.sh is running on $(hostname)"
echo "******************************************************************"
date > date_$EXP
rm -f ${SCRATCHDIR}/PROF.dat
YYYYMMDDHH=$YYYYMMDDHH_beg
YMDH_ftp_beg=$YYYYMMDDHH
while [ $YYYYMMDDHH -le $YYYYMMDDHH_end ]; do 
  time_beg=`date +%s -d ${YYYYMMDDHH::8}UTC${YYYYMMDDHH:8:2}`
  time_end=`date +%s -d ${YYYYMMDDHH_end::8}UTC${YYYYMMDDHH_end:8:2}`
  days=$(( (time_end - time_beg)/86400 + 1 ))
  [ $days -lt $days_per_job ] && days_per_job=$days
  
  YMDH_job_beg=$YYYYMMDDHH
  YMDH_job_end=`ADD_DATE.sh ${YYYYMMDDHH} +$((days_per_job-1))days`

  time_beg=`date +%s` 
  
  # get the files every $days_per_ftp 
  if [ $YMDH_job_beg -eq $YMDH_ftp_beg ]; then
      YMDH_ftp_end=`ADD_DATE.sh ${YMDH_ftp_beg} +$((days_per_ftp-1))days`
      [ $YMDH_ftp_end -gt $YYYYMMDDHH_end ] && YMDH_ftp_end=$YYYYMMDDHH_end
      # get forcings
      ./GET_FORCING.sh $YMDH_ftp_beg $YMDH_ftp_end $SCRATCHDIR/forcings $FORCINGDIR $days_FC
      # get observations
      [ "$TYP" != "OL"  -a "$TYP" != "ENOL" ] && ./GET_OBSERVATION.sh $YMDH_ftp_beg $YMDH_ftp_end $SCRATCHDIR/observations $OBSDIR
      # get restart files
      [ "$TYP" = "ENOL" ] && ./GET_RESTART.sh $BEG_ID:$END_ID $YMDH_ftp_beg $TYP:$LTRIP $INIDIR_ENOL $RUNDIR
  fi
  
  time_end=`date +%s` 
  echo "${YYYYMMDDHH} 0 $((time_end - time_beg))" >> ${SCRATCHDIR}/PROF.dat

  # run an ensemble with ID range from $BEG_ID to $END_ID
  rm -f finished_* .doing.* .redolist .todolist .*_0.nc SURFOUT*.nc batch_*
  echo "${YMDH_job_beg} ${BEG_ID}" > .todolist
  IJOB=0
  while [ $IJOB -lt $NJOB ]; do
    jobid=`sbatch ./BATCH.EXE $TYP:${tasks_for_trip} $BEG_ID:$END_ID ${YMDH_job_beg}:${YMDH_job_end}:${days_FC} $RUNDIR/data_oasis $SCRATCHDIR 2> /dev/null`
    jobfiles[$IJOB]="batch_`echo "$jobid" | awk '{print $4}'`"
    echo "$jobid for BATCH from ${YMDH_job_beg} to ${YMDH_job_end}"
    IJOB=$((IJOB+1))
  done
  
  # check if batch runs completed
  DT=1
  while true; do
    sleep $DT
    
    DOING=''
    NERR=0
    NFIN=0
    TOUT=0
    IJOB=0
    while [ $IJOB -lt $NJOB ]; do      
      if [ -f ${jobfiles[${IJOB}]} ]; then
        jobid=`echo ${jobfiles[${IJOB}]} | cut -d_ -f2`
        
        STATE1=`grep "#BATCH.EXE ${YMDH_job_end}#" ${jobfiles[${IJOB}]}`
        [ ! -z "$STATE1" ] && NFIN=$((NFIN+1)) && break

        STATE1=`grep 'State                : ' ${jobfiles[${IJOB}]}`
        STATE2=`echo "$STATE1" | grep 'State                : COMPLETED'`
        STATE3=`echo "$STATE1" | grep 'State                : TIMEOUT'`
        
        [ ! -z "$STATE2" ] && NFIN=$((NFIN+1)) && break
        if [ ! -z "$STATE1" ] && [ -z "$STATE2" ]; then
          [ -f .doing.${jobid} ] && DOING=`cat .doing.${jobid}`
          NERR=$((NERR+1))
        fi
        STATE1=`grep 'DUE TO TIME LIMIT' ${jobfiles[${IJOB}]}`
        if [ ! -z "$STATE3" ] || [ ! -z "$STATE1" ]; then
          TOUT=$((TOUT+1))
          if [ -f .doing.${jobid} ]; then
            DOING=`cat .doing.${jobid}`
            exec 333<>.redolist || continue
            flock 333 || continue
            REDO=`cat .redolist`
            [ "$REDO" == "" ] && echo $DOING >.redolist
            flock -u 333
            rm -f .doing.${jobid}
          fi
        fi
      fi
      IJOB=$((IJOB+1))
    done
    
    # Even some jobs are still in the queue, $NFIN greater than zero means the BATCH from $YMDH_job_beg to $YMDH_job_end is done
    if [ $NFIN -gt 0 ]; then
      # cancel other jobs
      IJOB=0
      while [ $IJOB -lt $NJOB ]; do
        jobid=`echo ${jobfiles[${IJOB}]} | cut -d_ -f2`
        if [ -f ${jobfiles[${IJOB}]} ]; then
          STATE1=`grep 'State                : ' ${jobfiles[${IJOB}]}`
          [ -z "$STATE1" ] && scancel $jobid 2>/dev/null
        else
          scancel $jobid 2>/dev/null
        fi
        IJOB=$((IJOB+1))
      done
      break
    elif [ $TOUT -eq $NJOB ]; then
      if [ ! -z "$DOING" ]; then
        NRUN=0
        [ "$TYP" = "ENKF" -o "$TYP" = "ENOL" ] && NRUN=$END_ID
        RESTART_DATE=`echo $DOING | awk '{print $1; exit}'`
        DOING_ID=`echo $DOING | awk '{print $2; exit}'`
        #[ $DOING_ID -eq $((END_ID+1)) ] && RESTART_DATE=`ADD_DATE.sh ${RESTART_DATE} -1day`
        YMDH_ftp_end=`ADD_DATE.sh ${RESTART_DATE} -1day`
        
        # upload RPOFILE
        date >> date_$EXP
        time_beg=`head -n1 date_$EXP`
        time_beg=`date +%s -d "$time_beg"`
        time_end=`tail -n1 date_$EXP`
        time_end=`date +%s -d "$time_end"`
        echo "0000000000 5 $((time_end - time_beg))" >> ${SCRATCHDIR}/PROF.dat
        lftp hendrix -e "put -c ${SCRATCHDIR}/PROF.dat -o $RESDIR/PROF_${YYYYMMDDHH_beg}-${RESTART_DATE}.dat; quit" 2>/dev/null
        # upload results
        ./PUT_RESULT.sh $BEG_ID:$NRUN $YMDH_ftp_beg:$YMDH_ftp_end:$RESTART_DATE $SCRATCHDIR:$RESDIR $TYP $days_FC 1

        # increase the job time limit by 10 minutes
        hh=`echo $job_time_limit | cut -d: -f1`
        mm=`echo $job_time_limit | cut -d: -f2`
        ss=`echo $job_time_limit | cut -d: -f3`
        mm=$((mm+10))
        if [ $mm -ge 60 ]; then
          mm=$((mm-60))
          hh=$((hh+1))
        fi
        job_time_limit="$(printf '%02d:%02d:%02d' $hh $mm $ss)"
        
        # restart
        logfile="$RUNDIR/${EXP}_${RESTART_DATE}.log"
        echo "TIMEOUT with the BATCH.EXE at $RESTART_DATE, we restart the running automatically!!!"
        echo "nohup $RUNDIR/script_ldas.sh $TYP:$NJOB:${days_FC} $DOM:$EXP $RESTART_DATE:$YYYYMMDDHH_end $days_per_job:$days_per_ftp $num_nodes:$tasks_per_node:$tasks_for_trip $job_time_limit $VERSION >& $logfile &"
        cd $RUNDIR
        nohup $RUNDIR/script_ldas.sh $TYP:$NJOB:${days_FC} $DOM:$EXP $RESTART_DATE:$YYYYMMDDHH_end $days_per_job:$days_per_ftp $num_nodes:$tasks_per_node:$tasks_for_trip $job_time_limit $VERSION >& $logfile &
      else
        echo "TIMEOUT with the BATCH.EXE, please enlarge the commandline argument job_time_limit!!!"
      fi
      exit 1
    elif [ $NERR -eq $NJOB ]; then
      if [ ! -z "$DOING" ]; then
        NRUN=0
        [ "$TYP" = "ENKF" -o "$TYP" = "ENOL" ] && NRUN=$END_ID
        RESTART_DATE=`echo $DOING | awk '{print $1; exit}'`
        DOING_ID=`echo $DOING | awk '{print $2; exit}'`
        #[ $DOING_ID -eq $((END_ID+1)) ] && RESTART_DATE=`ADD_DATE.sh ${RESTART_DATE} -1day`
        YMDH_ftp_end=`ADD_DATE.sh ${RESTART_DATE} -1day`

        # upload RPOFILE
        date >> date_$EXP
        time_beg=`head -n1 date_$EXP`
        time_beg=`date +%s -d "$time_beg"`
        time_end=`tail -n1 date_$EXP`
        time_end=`date +%s -d "$time_end"`
        echo "0000000000 5 $((time_end - time_beg))" >> ${SCRATCHDIR}/PROF.dat
        lftp hendrix -e "put -c ${SCRATCHDIR}/PROF.dat -o $RESDIR/PROF_${YYYYMMDDHH_beg}-${RESTART_DATE}.dat; quit" 2>/dev/null
        # upload results
        ./PUT_RESULT.sh $BEG_ID:$NRUN $YMDH_ftp_beg:$YMDH_ftp_end:$RESTART_DATE $SCRATCHDIR:$RESDIR $TYP $days_FC

        echo "Error with the BATCH.EXE at $RESTART_DATE, please check EXP/$EXP/batch_*!!!"
      else
        echo "Error with the BATCH.EXE, please check EXP/$EXP/batch_*!!!"
      fi
      exit 1
    fi
  done
  
  time_beg=`date +%s` 
  echo "${YYYYMMDDHH} 3 $((time_beg - time_end))" >> ${SCRATCHDIR}/PROF.dat

  # save results
  if [ $YMDH_job_end -eq $YMDH_ftp_end ]; then
    NRUN=0
    [ "$TYP" = "ENKF" -o "$TYP" = "ENOL" ] && NRUN=$END_ID
    RESTART_DATE=`ADD_DATE.sh ${YMDH_ftp_end} +1day`
    # move the restart files
    IRUN=$BEG_ID
    while [ $IRUN -le $NRUN ]; do  
      cp -f PREP_$IRUN.nc $SCRATCHDIR/PREP_$IRUN.nc
      if [ $LTRIP -eq 1 ]; then
        cp -f TRIP_PREP_$IRUN.nc $SCRATCHDIR/TRIP_PREP_$IRUN.nc
        cp -f lsm_$IRUN.nc $SCRATCHDIR/lsm_$IRUN.nc
        cp -f trip_$IRUN.nc $SCRATCHDIR/trip_$IRUN.nc
      fi
      IRUN=$((IRUN+1))
    done
    ./PUT_RESULT.sh $BEG_ID:$NRUN $YMDH_ftp_beg:$YMDH_ftp_end:$RESTART_DATE $SCRATCHDIR:$RESDIR $TYP $days_FC
    YMDH_ftp_beg=`ADD_DATE.sh ${YMDH_ftp_end} +1day`
  fi

  # jobs are done but they may still be in the queue, move the output files only when they exit the queue completely
  IJOB=0
  while [ $IJOB -lt $NJOB ]; do
    jobid=`echo ${jobfiles[${IJOB}]} | cut -d_ -f2`
    DT=1; while [ ! -z "`squeue --job $jobid 2> /dev/null | grep $jobid`" ]; do sleep $DT; done
    [ -f ${jobfiles[${IJOB}]} ] && mv -f ${jobfiles[${IJOB}]} oe/batch_${YYYYMMDDHH}.${IJOB}
    IJOB=$((IJOB+1))
  done

  time_end=`date +%s` 
  echo "${YYYYMMDDHH} 4 $((time_end - time_beg))" >> ${SCRATCHDIR}/PROF.dat
  
  YYYYMMDDHH=`ADD_DATE.sh ${YMDH_job_end} +1day`
done
date >> date_$EXP
time_beg=`head -n1 date_$EXP`
time_beg=`date +%s -d "$time_beg"`
time_end=`tail -n1 date_$EXP`
time_end=`date +%s -d "$time_end"`
echo "0000000000 5 $((time_end - time_beg))" >> ${SCRATCHDIR}/PROF.dat
lftp hendrix -e "put -c ${SCRATCHDIR}/PROF.dat -o $RESDIR/PROF_${YYYYMMDDHH_beg}-${YYYYMMDDHH_end}.dat; quit" 2>/dev/null


echo "***************************************"
echo "   SIMULATION ENDS CORRECTLY"
echo "***************************************"

